import json
import os

from config import *

data_path = 'data'
data_TEST_path = 'data_TEST'
minecraft_path = 'minecraft'
main_path = 'main'

def get_name(name=str) -> str:
    total = name[0].upper()
    for i in range(1, len(name)):
        if name[i] == '_': total += ' '
        else: total += name[i]

    return total

def create_setblock_item_json(name=str, path=str) -> None:
    data = {
        "pools": [
            {
                "rolls": 1,
                "entries": [
                    {
                        "type": "minecraft:item",
                        "name": "minecraft:knowledge_book",
                        "functions": [
                            {
                                "function": "minecraft:set_name",
                                "name": {
                                    "text": f'DROP ME NOW ({name})',
                                    "color": "red",
                                    "italic": False
                                }
                            },
                            {
                                "function": "minecraft:set_nbt",
                                "tag": "{"+f'setblock_bool:True,setblock_id:{name}'+"}"
                            }
                        ]
                    }
                ]
            }
        ]
    }
    with open(f'{path}/{name}.json', "w") as file:
        json.dump(data, file)
    with open(f'{data_path}/stage_break/functions/setblock_item_check.mcfunction', 'a') as file:
        file.write('execute if entity @s[nbt={Item:{tag:{setblock_id:'+name+'}}}] run setblock ~ ~ ~ '+name+'\n')
        file.write('execute if entity @s[nbt={Item:{tag:{setblock_id:'+name+'}}}] run kill @s\n\n')


def condition_create(alt, child):
    try:
        if alt['condition'][0] == '!player':
            child['conditions'] = [
                {
                    'condition': 'inverted',
                    'term': {
                        'condition': 'entity_properties',
                        'entity': 'this',
                        'predicate': {'type': 'player'}
                    }

                }
            ]
        elif alt['condition'][0] == 'match_tool':
            child['conditions'] = [
                {
                    'condition': 'match_tool',
                    'predicate': alt['condition'][1]
                }  
            ]
    except: None
    return child

def count_create(alt, child):
    try:
        if type(alt['count']) == str:
            child['functions'] = [
	        	{
	        		'function': 'minecraft:set_count',
	        		'count': alt['count']
	        	}
	        ]
        elif type(alt['count']) != str:
            child['functions'] = [
	        	{
	        		'function': 'minecraft:set_count',
	        		'count': alt['count']
	        	}
	        ]
    except: None
    return child

def function_create(alt, child):
    try:
        function = alt['function'][1]
        function['function'] = alt['function'][0]
        child['functions'].append(function)
    except: None
    return child

def create_roll_list_json(name=str, drop=list, prints=False):
    data = {
	    'pools': [

        ]
    }
    # if prints: print(f'{name}:')
    for alt in drop:
        try:
            data["pools"].append(
                {
	            	'rolls': 1,
	            	'entries': [
	            		child_create(alt=alt, prints=prints, name=name)
                    ]
                }
                
            )
        except: print(f'\n\n\nERROR:\n{alt}\n')
    with open(f'{data_path}/stage_break/loot_tables/roll_list/{name}.json', "w") as file:
        json.dump(data, file)

def child_create(alt, prints=False, name=str):
    try:
        item = alt['item']
        child = {
            'type': 'item',
            'name': item
        }
        child = count_create(alt=alt, child=child)
        child = function_create(alt=alt, child=child)
        child = condition_create(alt=alt, child=child)

        if prints: print('\nitem:', alt)
    except: None
    try:
        alternatives = alt['alternatives']
        child = {
            'type': 'alternatives',
            'children': []
        }
        for alternatives_child in alternatives:
            child['children'].append(
                child_create(alternatives_child, name=name)
            )
        child = count_create(alt=alt, child=child)
        child = function_create(alt=alt, child=child)
        child = condition_create(alt=alt, child=child)

        if prints: print('\nsequence', alt)
    except: None
    try:
        stage_break = alt['stage_break']
        child = {
            'type': 'loot_table',
            'name': f'stage_break:setblock/{stage_break}'
        }
        child = count_create(alt=alt, child=child)
        child = function_create(alt=alt, child=child)
        child = condition_create(alt=alt, child=child)

        create_setblock_item_json(name=stage_break, path=f'{data_path}/stage_break/loot_tables/setblock')

        if prints: print('\nstage_break', alt)
    except: None
    try:
        roll_list = alt['roll_list']
        if prints: print('\nroll_list', alt)
        child = {
            'type': 'loot_table',
            'name': f'stage_break:roll_list/{name}'
        }

        create_roll_list_json(
            name=name,
            drop=roll_list,
            prints=prints
        )

        child = count_create(alt=alt, child=child)
        child = function_create(alt=alt, child=child)
        child = condition_create(alt=alt, child=child)

    except: None
    return child

def create_block_drop_json(name=str, drop=list, prints=False):
    data = {
	    'type': 'block',
	    'pools': [
	    	{
	    		'rolls': 1,
	    		'entries': [
	    			{
	    				'type': 'alternatives',
	    				'children': [

                        ]
                    }
                ]
            }
        ]
    }
    if prints: print(f'{name}:')
    for alt in drop:
        try:
            data["pools"][0]['entries'][0]['children'].append(
                child_create(alt=alt, prints=prints, name=name)
            )
        except: print(f'\n\n\nERROR:\n{alt}\n')
    with open(f'{data_path}/{minecraft_path}/loot_tables/blocks/{name}.json', "w") as file:
        json.dump(data, file)

def create_craft_json(result, ingredients, path):
    data = {
        "type": "crafting_shapeless",
        "ingredients": [{'item': ingredient} for ingredient in ingredients],
        "result": {
            "item": result
        }
    }
    with open(f'{path}/{result}.json', "w") as file:
        json.dump(data, file)

def main():
    print('STARTED block drops')

    with open(f'{data_path}/stage_break/functions/setblock_item_check.mcfunction', "w") as file: file.write("")

    if os.path.exists(f'{data_path}/stage_break/loot_tables/roll_list'):
        os.system(f'rm -rf {data_path}/stage_break/loot_tables/roll_list')
    os.makedirs(f'{data_path}/stage_break/loot_tables/roll_list')

    if os.path.exists(f'{data_path}/stage_break/loot_tables/setblock'):
        os.system(f'rm -rf {data_path}/stage_break/loot_tables/setblock')
    os.makedirs(f'{data_path}/stage_break/loot_tables/setblock')

    if os.path.exists(f'{data_path}/{minecraft_path}/loot_tables/blocks'):
        os.system(f'rm -rf {data_path}/{minecraft_path}/loot_tables/blocks')
    os.makedirs(f'{data_path}/{minecraft_path}/loot_tables/blocks')



    for drop in custom_block_drops:
        create_block_drop_json(name=drop, drop=custom_block_drops[drop], prints=False)

    print('COMPLETE block drops')
    print('STARTED crafts')
    for result in custom_crafts:
        create_craft_json(result=result, ingredients=custom_crafts[result], path=f'{data_path}/{minecraft_path}/recipes')
    print('COMPLETE crafts')

main()